/*
	SimpleView
	views some .tga .qrt .gif in individual windows
*/

#include "SVWindow.h"
#include <interface/View.h>
#include <interface/MenuItem.h>
#include <app/Application.h>
#include <support/Debug.h>
#include <support/SupportDefs.h>
#include <kernel/OS.h>

#include "SVApplication.h"
extern SVApplication *sv_app;

/* SVWindow constructor :
WARNING ! we take responsability for the BBitmap object, we will delete it
ourselves
2nd warning, we use a type 11 window, with a resize tab. Thanks G.E. !
*/
SVWindow::SVWindow( char *title, BBitmap *p )
	: BWindow( BRect( 0.0f, 0.0f, 0.0f, 0.0f ), title, (window_type)11, 0l )
{
// we want secured operations
	SetDiscipline( TRUE );
	w = p->Bounds( ).Width( );
	h = p->Bounds( ).Height( );
// we set up a BScrollView which will draw our BBitmap
// we calculate an appropriate size for the blit_view : at most 80% of the
// screen size and top left corner at 10%

// get screen width and height
	screen_info screen;
	get_screen_info( &screen );
	float sw = 0.80f*screen.frame.Width( ), sh = 0.80f*screen.frame.Height( );

// build the target (scrolled bitmap) view and the scrollview, attach them
	blit_view = new SVView( p ); ASSERT( blit_view!=0 );
	scroll_view = new BScrollView( "SVScrollView", blit_view,
	 B_FOLLOW_ALL, 0, TRUE, TRUE, FALSE ); ASSERT( scroll_view!=0 );

// get the scrollview size, resize the window accordingly
// yes the BScrollBars are actually 2 dots too large. This leaves crap on
// scrolling, and the type 11 resizing tab won't fit the free area.
	Lock( );
	float dw = scroll_view->Frame( ).Width( )-2.0f,
	 dh = scroll_view->Frame( ).Height( )-2.0f;
	Unlock( );
// first we set the window size as if it had the full-sized scrollview
	ResizeTo( dw, dh );
// then we stick the scroll view to the window which is at its full size
	AddChild( scroll_view );
// then we eventually size down the window so that it fits in the screen
	ResizeTo( min( dw, sw ), min( dh, sh ) );

	MoveTo( screen.frame.left+0.10f*sw, screen.frame.top+0.10f*sh );

// This function refers to the TOTAL size, title tab included.
// Looks a bit odd, as we don't know its size.
	SetSizeLimits( 19.0f, dw+8.0f, 37.0f, dh+27.0f );	
	Show( );	
}

/* FrameResized :
We set the scrollbars range and steps according to the new size.
*/
void SVWindow::FrameResized( float width, float height )
{
	Lock( );
	BScrollBar *h_bar = scroll_view->ScrollBar( B_HORIZONTAL ),
	 *v_bar = scroll_view->ScrollBar( B_VERTICAL );
	float new_w = blit_view->Frame( ).Width( ),
	 new_h = blit_view->Frame( ).Height( );
	h_bar->SetRange( 0l, ( long )( w-new_w ) );
	v_bar->SetRange( 0l, ( long )( h-new_h ) );
	h_bar->SetSteps( 1l, ( long )new_w );
	v_bar->SetSteps( 1l, ( long )new_h );
	Unlock( );
}

/* ScrollBy :
We scroll the bars by the given offset.
This is a support function for the bitmap dragging.
*/
void SVWindow::ScrollBy( BPoint offset )
{
	Lock( );
	BScrollBar *h_bar = scroll_view->ScrollBar( B_HORIZONTAL ),
	 *v_bar = scroll_view->ScrollBar( B_VERTICAL );
	h_bar->SetValue( h_bar->Value( )+( long )offset.x );
	v_bar->SetValue( v_bar->Value( )+( long )offset.y );
	Unlock( );
}

/* QuitRequested :
We return false and post a B_QUIT_REQUESTED msg if it's an attempt to close
the last window but not from the application itself trying to quit.
Otherwise we agree to quit.
*/
bool SVWindow::QuitRequested( )
{
	if( sv_app->NumPic( )==1 )
		sv_app->PostMessage( B_QUIT_REQUESTED );
	sv_app->RemPic( this );
	return TRUE;
}

/*  SVView constructor :
We take the responsibility for the BBitmap object !
*/
SVView::SVView( BBitmap *p )
	: BView( p->Bounds( ), "SVView", B_FOLLOW_ALL, B_WILL_DRAW )
{
	pic = p;
	mouse_scroll = FALSE;
}

SVView::~SVView( )
{
	delete pic;
}

/* Draw :
Simple uh ? Here appears the power of the Be interface kit !
We redraw the asked rectangle. It is given in BView bounds, that's why it's
so easy.
*/
void SVView::Draw( BRect update )
{
	DrawBitmap( pic, update, update );
}

/* MouseDown :
We start tracking for the mouse and dragging the bitmap accordingly.
For button=2 (right button) we display the app menu as a pop-up menu
If the user selects "pictures" the returned BMenuItem won't have a BMessage
*/
void SVView::MouseDown( BPoint point )
{
	BPoint p;
	ulong button;
	GetMouse( &p, &button, FALSE );
	switch( button ) {
	case B_PRIMARY_MOUSE_BUTTON:
		mouse_scroll = TRUE;
		ConvertToScreen( &point );
		mouse_track = point;
		break;
	case B_SECONDARY_MOUSE_BUTTON:
		ConvertToScreen( &point );
		BMessage *do_popup = new BMessage( DO_POPUP_MENU ); ASSERT( do_popup!=0 );
		do_popup->AddPoint( "where", point );
		sv_app->PostMessage( do_popup );
		break;
	case B_TERTIARY_MOUSE_BUTTON:
		;
	}
}

/* MouseMoved :
If a drag is on the run, we scroll the bars. This will redraw the window.
The advantages of this way :
- it's event driven : less cpu intensive (check with Pulse !) and cleaner
- the window can receive other messages will the drag. Say there is a window
	moving by itself in front of ours. This way our window can be dragged
	and still redrawn as necessary !
*BAD NEWS* the msgs seem not to arrive in the right order :-(
It appears that the update messages generated mess up the incoming stream
of mouse moved messages.
Personnal note : I HATE the suggested usage of BeginRectTracking( ) !
*/
void SVView::MouseMoved( BPoint point, ulong transit, BMessage *msg )
{
	if( transit!=B_INSIDE_VIEW ) {
		mouse_scroll = FALSE;
		return ;
	} else
		if( mouse_scroll ) {
			ConvertToScreen( &point );
			( ( SVWindow * )Window( ) )->ScrollBy( mouse_track-point );
			mouse_track = point;
		}		
}

/* MouseUp :
We've finished the drag.
*/
void SVView::MouseUp( BPoint point )
{
	mouse_scroll = FALSE;
}

/* MessageDropped :
*/
bool SVView::MessageDropped( BMessage *msg, BPoint where, BPoint offset )
{
//	msg->PrintToStream( );
//	record_ref ref;
//	if( msg->HasRef( "refs" ) ) {
//		ref = msg->FindRef( "refs" );
//		BMessage *op = new BMessage( B_REFS_RECEIVED );
//		op->AddRef( "refs", ref );
//		sv_app->PostMessage( op );
//	}
	return FALSE;
}

/* MessageReceived :
As B_MOUSE_UP events don't have an associated hook function in the BView class
we do it by ourselves. Of course the call to the original MessageReceived
is useless here, but anyway...
*/
void SVView::MessageReceived( BMessage *msg )
{
	switch( msg->what ) {
	case B_MOUSE_UP:
		MouseUp( msg->FindPoint( "where" ) );
		break;
	default:
		BView::MessageReceived( msg );
	}
}

//  Benot Triquet 1996